<?php

# This program is in the public domain within the United States. Additionally,
# we waive copyright and related rights in the work worldwide through the CC0
# 1.0 Universal public domain dedication, which can be found at
# http://creativecommons.org/publicdomain/zero/1.0/


# This file is for running unit tests. Either on wfpl or your own code.

require_once(__DIR__.'/'.'template.php');
require_once(__DIR__.'/'.'messages.php');

# call this to declare that a unit test has passed
function unit_test_passed($msg) {
	$GLOBALS['unit_tests_passed_msgs'][] = $msg;
	$GLOBALS['unit_tests_passed'] += 1;
}

# call this to declare that a unit test has failed
function unit_test_failed($msg) {
	$GLOBALS['unit_tests_failed_msgs'][] = $msg;
	$GLOBALS['unit_tests_failed'] += 1;
}

function backslashinate($str) {
	return '"' . addcslashes($str, "\0..\37\"\\\177..\377") . '"';
}

function backslashinate_each($a) {
	foreach($a as &$v) {
		$v = backslashinate($v);
	}

	return $a;
}
	

# use this function to unit-test a function
# real prototype: ($name, $args..., $correct)
function unit_test_func() {
	$args = func_get_args();
	$function = array_shift($args);
	$correct = array_pop($args);
	$result = call_user_func_array($function, $args);
	$message = "$function(" . join(', ', backslashinate_each($args)) . ") returned " . backslashinate($result);
	if($result === $correct) {
		unit_test_passed($message);
	} else {
		unit_test_failed($message . " instead of " . backslashinate($correct));
	}
}


function file_run_unit_tests($filename) {
	require_once($filename);
	$func = basename($filename, '.php') . '_unit_tests_main';

	if(function_exists($func)) {
		return $func();
	}
}

# Run many unit tests. Pass a directory containing php files with tests in
# them, and a space-separated list of their basenames. Each of those files is
# expected to define a function <basename>_unit_tests_main which will run all
# tests in that file. It should print a message() about each test that failed,
# and return the number of tests that failed.
function run_unit_tests($directory, $basenames) {
	$files = array();
	$GLOBALS['unit_tests_passed'] = 0;
	$GLOBALS['unit_tests_failed'] = 0;
	$basenames = explode(' ', $basenames);
	foreach($basenames as $basename) {
		$GLOBALS['unit_tests_passed_msgs'] = array();
		$GLOBALS['unit_tests_failed_msgs'] = array();
		$already_passed = $GLOBALS['unit_tests_passed'];
		$already_failed = $GLOBALS['unit_tests_failed'];
		$filename = "$directory/$basename.php";
		file_run_unit_tests($filename);
		$files[] = array(
			'file' => $filename,
			'fails' => columnize($GLOBALS['unit_tests_failed_msgs']),
			'passes' => columnize($GLOBALS['unit_tests_passed_msgs']),
			'count_passed' => $GLOBALS['unit_tests_passed'] - $already_passed,
			'count_failed' => $GLOBALS['unit_tests_failed'] - $already_failed
		);
	}
	$passed = $GLOBALS['unit_tests_passed'];
	$failed = $GLOBALS['unit_tests_failed'];
	tem_set('unit_tests', array(
		'files' => $files,
		'total_passed' => $passed,
		'total_failed' => $failed
	));
}

# Call this to unit test wfpl. By default it tests everything, or you can pass
# a space-separated list of the basenames of the files in wfpl/unit_tests/
# that you'd like to run.
function unit_test_wfpl($basenames = 'format db misc encode') {
	tem_load(__DIR__.'/'.'unit_tests/template.html');
	run_unit_tests(__DIR__.'/'.'unit_tests', $basenames);
	display_messages();
	tem_output();
	exit();
}
